/**************************************************************************************
Author:			David Szybala
Website:        www.davidszybala.com
Alt Website:   www.twistedgerbil.com
Program:        Calculator.ms
Date:            2/10/09
Description:   This script will create a simple calculator.
***************************************************************************************/

try (destroyDialog calculator_rollout) catch()


(--put script into local scope
    global calculator_rollout
    
    local valid_chars = #("1","2","3","4","5","6","7","8","9","0","(",")","+","-","*","/"," ")
    local current_equation = ""
    local calculator_ini_file = getDir #plugcfg + "\\Calculator.ini"
    local calculator_rollout_pos
    
    /************************************************************************
    Returns the result
    ************************************************************************/
    fn set_equation_result the_equation =
    (
        local result = 0
        
        try
            result = execute the_equation
        catch
            result = "ERROR"
        
        calculator_rollout.equation_text.text = (result as string)
    ) --end fn
    
    /************************************************************************
    UI Creation
    ************************************************************************/
    rollout calculator_rollout "Calculator"
    (
        edittext equation_text "" pos:[2,5] width:148 height:17 text:"" readOnly:false 
        
        button number_one_button "1" pos:[5,30] width:20 height:20
        button number_two_button "2" pos:[30,30] width:20 height:20
        button number_three_button "3" pos:[55,30] width:20 height:20
        button number_four_button "4" pos:[5,55] width:20 height:20
        button number_five_button "5" pos:[30,55] width:20 height:20
        button number_six_button "6" pos:[55,55] width:20 height:20
        button number_seven_button "7" pos:[5,80] width:20 height:20
        button number_eight_button "8" pos:[30,80] width:20 height:20
        button number_nine_button "9" pos:[55,80] width:20 height:20
        button negative_button "-" pos:[5,105] width:20 height:20
        button number_zero_button "0" pos:[30,105] width:20 height:20
        button decimal_button "." pos:[55,105] width:20 height:20
        
        button clear_button "CE" pos:[105,30] width:20 height:20 toolTip:"Clear Input"
        button clear_all_button "C" pos:[130,30] width:20 height:20 toolTip:"Clear All"
        button multiply_button "*" pos:[105,55] width:20 height:20
        button divide_button "/" pos:[130,55] width:20 height:20
        button subtract_button "-" pos:[105,80] width:20 height:20
        button add_button "+" pos:[130,80] width:20 height:20    
        button equals_button "=" pos:[130,105] width:20 height:20
        
        /**********************************************************************
        UI move event handler. Stores the current position of the UI
        **********************************************************************/
        on calculator_rollout moved pos do
            setIniSetting calculator_ini_file "Dialog" "Position" (pos as string)
        
        /**********************************************************************
        Equation changed event handler. Cleans the text entered by the user
        **********************************************************************/
        on equation_text changed the_text do
        (
            local clean_text = ""
            
            for i = 1 to the_text.count do
            (
                if (findItem valid_chars the_text[i]) > 0 do
                    clean_text += the_text[i]
            ) --end for
            
            equation_text.text = clean_text
        ) --end on
        
        /**********************************************************************
        Equation enter event handler. Returns the result when the enter key is pressed
        **********************************************************************/
        on equation_text entered the_text do
        (
            set_equation_result (current_equation + the_text)
            current_equation = ""
        ) --end on
        
        /**********************************************************************
        Number clicked event handler. Appends the selected number to the equation
        **********************************************************************/
        on number_one_button pressed do equation_text.text += "1"
        on number_two_button pressed do equation_text.text += "2"
        on number_three_button pressed do equation_text.text += "3"
        on number_four_button pressed do equation_text.text += "4"
        on number_five_button pressed do equation_text.text += "5"
        on number_six_button pressed do equation_text.text += "6"
        on number_seven_button pressed do equation_text.text += "7"
        on number_eight_button pressed do equation_text.text += "8"
        on number_nine_button pressed do equation_text.text += "9"
        on number_zero_button pressed do equation_text.text += "0"
        
        /**********************************************************************
        Negative clicked event handler. Appends a negative sign to the equation. WARNING: Do NOT
        allow two negative signs together
        **********************************************************************/
        on negative_button pressed do
        (
            local cleaned_text = substituteString equation_text.text " " ""
            
            if cleaned_text.count > 0 do
            (
                if cleaned_text[cleaned_text.count] != "-" do
                    equation_text.text += "-"
            ) --end if
        ) --end on
        
        /**********************************************************************
        Decimal clicked event handler. Appends a decimal to the equation
        **********************************************************************/
        on decimal_button pressed do equation_text.text += "."
        
        /**********************************************************************
        Clear button clicked event handler. Clears the current equation text
        **********************************************************************/
        on clear_button pressed do equation_text.text = ""
        
        /**********************************************************************
        Clear all button clicked event handler. Clears the current and stored equation
        **********************************************************************/
        on clear_all_button pressed do equation_text.text = current_equation = ""
        
        /**********************************************************************
        Multiply button clicked event handler. Applies a multiply operation if the equation is valid
        **********************************************************************/
        on multiply_button pressed do
        (
            if equation_text.text != "" do
            (
                current_equation = (current_equation + equation_text.text + " * ")
                equation_text.text = ""
            ) --end if
        ) --end on
        
        /**********************************************************************
        Divide button clicked event handler. Applies a divide operation if the equation is valid
        **********************************************************************/
        on divide_button pressed do
        (
            if equation_text.text != "" do
            (
                current_equation = (current_equation + equation_text.text + " / ")
                equation_text.text = ""
            ) --end if
        ) --end on
        
        /**********************************************************************
        Subtract button clicked event handler. Applies a subtract operation if the equation is valid
        **********************************************************************/
        on subtract_button pressed do
        (
            if equation_text.text != "" do
            (
                current_equation = (current_equation + equation_text.text + " - ")
                equation_text.text = ""
            ) --end if
        ) --end on
        
        /**********************************************************************
        Add button clicked event handler. Applies a subtract operation if the equation is valid
        **********************************************************************/
        on add_button pressed do
        (
            if equation_text.text != "" do
            (
                current_equation = (current_equation + equation_text.text + " + ")
                equation_text.text = ""
            ) --end if
        ) --end on
        
        /**********************************************************************
        Equals button clicked event handler. Displays the result of the equation
        **********************************************************************/
        on equals_button pressed do
        (
            set_equation_result (current_equation + equation_text.text)
            current_equation = ""
        ) --end on
    ) --end rollout
    
    calculator_rollout_pos = execute (getIniSetting calculator_ini_file "Dialog" "Position")
    if calculator_rollout_pos == OK do calculator_rollout_pos = [100,100]
    if calculator_rollout_pos.x > sysinfo.desktopsize.x - 155 do
        calculator_rollout_pos.x = sysinfo.desktopsize.x - 160
    if calculator_rollout_pos.y > sysinfo.desktopsize.y - 190 do
        calculator_rollout_pos.y = sysinfo.desktopsize.y - 190
    if calculator_rollout_pos.x < 0 do calculator_rollout_pos.x = 0 
    if calculator_rollout_pos.y < 0 do calculator_rollout_pos.y = 0
    createDialog calculator_rollout 155 130 calculator_rollout_pos.x calculator_rollout_pos.y
) --end local scope
